/*
    KiFi, KDE Wireless Manager
    Copyright (c) 2002-2004 by Chris TenHarmsel <chris@staticmethod.net>

    KiFi, (c) 2004 by the KiFi developers

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "interfacemonitor.h"
#include <qfile.h>
#include <kdebug.h>
#include <iostream>

// static definition of proc file
const QString InterfaceMonitor::PROC_FILE("/proc/net/wireless");

InterfaceMonitor::InterfaceMonitor( QObject *parent, const char* name )
	: QObject( parent, name )
{
	_timer = new QTimer( this );
	// Connect the timer to the update method
	QObject::connect( _timer, SIGNAL( timeout() ),
										this, SLOT( updateInfo() ) );
	_timerInterval = 5000;
}

InterfaceMonitor::~InterfaceMonitor()
{
}

void InterfaceMonitor::start()
{
	_timer->start(_timerInterval);
}

void InterfaceMonitor::stop()
{
	_timer->stop();
}

void InterfaceMonitor::updateInfo()
{
	QString line;
	QStringList readInterfaces;
	QFile file(PROC_FILE);

	if (!file.exists()) {
		// no file = no interfaces
		return;
	}

	if (!file.open(IO_ReadOnly)) {
		// can't read file
		return;
	}

	//
	// eat first two lines of header info
	//
	if (file.readLine(line, 1024) <= 0)
	{
		// either couldn't read or read nothing
		// either way, it's not as expected
		return;
	}
	if (file.readLine(line, 1024) <= 0)
	{
		// either couldn't read or read nothing
		// either way, it's not as expected
		return;
	}

	// as long as there are more lines
	while (-1 != file.readLine(line, 1024)) {
		// Add the interface name with no leading whitespace.
		// The name may contain ':' characters so we include all
		// chars until the last ':' in the line before trimming.
		readInterfaces.push_back(line.left(line.findRev(':')).stripWhiteSpace());
	}

	// close the file
	file.close();

	// for each interface we read
	QStringList::iterator it;
	for ( it = readInterfaces.begin(); it != readInterfaces.end(); ++it ) {
		if (!_knownInterfaces.contains(*it)) {
			// we didn't know about this interface
			_knownInterfaces.push_back(*it);
			// found one
			kdDebug() << "found interface [" << *it << "]" << endl;
			emit interfaceFound(*it);
		}
	}

	QStringList lostInterfaces;
	// for each interface we think exists
	for ( it = _knownInterfaces.begin(); it != _knownInterfaces.end(); ++it ) {
		if (!readInterfaces.contains(*it)) {
			// this interface is no longer with us
			lostInterfaces.push_back(*it);
			// lost one
			kdDebug() << "lost interface [" << *it << "]" << endl;
			emit interfaceLost(*it);
		}
	}

	// remove any interfaces we may have lost
	for ( it = lostInterfaces.begin(); it != lostInterfaces.end(); ++it ) {
		_knownInterfaces.remove(*it);
	}
}

#include "interfacemonitor.moc"

/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 2
 * indent-tabs-mode: t
 * End:
 */
