/*
    KiFi, KDE Wireless Manager
    Copyright (c) 2002-2004 by Chris TenHarmsel <chris@staticmethod.net>

    KiFi, (c) 2004 by the KiFi developers

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef _INTERFACEMONITOR_H_
#define _INTERFACEMONITOR_H_

#include <qobject.h>
#include <qstringlist.h>
#include <qtimer.h>

/**
 * This class monitors wireless device interfaces by reading /proc/net/wireless 
 * emitting signals upon discovery of a new interface and removal of an 
 * interface.
 */
class InterfaceMonitor : public QObject
{
	Q_OBJECT
public:
	/** Constructor */
	InterfaceMonitor(QObject *parent = 0, const char* name = 0);

	/** Destructor */
	virtual ~InterfaceMonitor();

	/**
	 * This method instructs the InterfaceMonitor to begin monitoring
	 * /proc/net/wireless for new interfaces
	 */
	void start();

	/**
	 * This method instructs the InterfaceMonitor to stop monitoring
	 * /proc/net/wireless for new interfaces
	 */
	void stop();

signals:
	/**
	 * This signal indicates that a new wireless interface has been
	 * detected.
	 *
	 * @param ifname name of the interface found.
	 */
	void interfaceFound( QString ifname );

	/**
	 * This signal indicates that wireless interface has been lost.
	 * 
	 * @param ifname name of the interface lost.
	 */
	void interfaceLost( QString ifname );

protected slots:
	/** 
	 * This method updates the interface list and emits signals
	 * if a change is found.
	 */
	void updateInfo();
	
protected:
	/** 
	 * Milliseconds to wait between each updating of state of 
	 * wireless devices and emitting signals.
	 */
	int _timerInterval;
	/** List of wireless interfaces that currently known to exist */
	QStringList _knownInterfaces;
	/** Timer to determine how often to poll for current interface state */
	QTimer *_timer;
	/** File to parse wireless interface names from */
	static const QString PROC_FILE;

};


#endif
/*
 * Local variables:
 * c-indentation-style: k&r
 * c-basic-offset: 2
 * indent-tabs-mode: t
 * End:
 */
