/*
    KiFi, KDE Wireless Manager
    Copyright (c) 2002-2004 by Chris TenHarmsel <chris@staticmethod.net>

    KiFi, (c) 2004 by the KiFi developers

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef _KWIRELESSMANAGERENGINE_H_
#define _KWIRELESSMANAGERENGINE_H_

#include <qwidget.h>
#include <qstring.h>

extern "C"{
// Private copy
#include <iwlib.h>
}

/**
 * This is the class that handles gathering info on a single
 * wireless interface, such as SSID and signal strength.
 * @author Christopher TenHarmsel <chris@staticmethod.net>
 **/
class WirelessInfoEngine : public QObject
{
  Q_OBJECT

public:
	WirelessInfoEngine(
		QString interface, QObject *parent = 0, const char *name = 0);
	~WirelessInfoEngine();

	/* Modes the card may be in */
	enum WirelessMode {Unknown, Auto, AdHoc, Infra};

	/**
	 * Gets the current signal quality
	 * @return The signal quality
	 */
	int getSignalQuality();

	/**
	 * Gets the signal level
	 * @return The signal level
	 */
	int getSignalLevel();

	/**
	 * Gets the signal noise
	 * @return The signal noise
	 */
	int getSignalNoise();

	/**
	 * Returns whether or not WEP is enabled.
	 */
	bool wepEnabled();

	/**
	 * Returns the SSID.  Don't forget to call updateInfo();
	 */
	QString getSSID();

	/**
	 * This function is used to tell the engine to update it's info
	 * from the wireless card.  It will query the card and then other
	 * functions will be able to retrieve this information.
	 */
	bool updateInfo();

	/**
	 * Gets the current WEP Key used.
	 */
	QString getWEPKey();

	/**
	 * Gets the current operating mode for the card
	 */
	int getMode();

protected: // Protected attributes

	/**
	 * Updates the signal stats
	 */
	bool  updateSignalStats();

	/**
	 * Convenience method to convert QString objects into
	 * unsigned char * objects
	 * @param string The string to convert
	 * @param uCharArray Pointer to the unsigned char array to copy into
	 * @param size The size of the char aray
	 */
	void qStringToUChar(QString string, unsigned char *uCharArray, int size);

	/**
	 * Convenience method to convert QString objects into
	 * char * objects
	 * @param string The string to convert
	 * @param charArray Pointer to the char array to copy into
	 * @param size The size of the char aray
	 */
	void qStringToChar(QString string, char *charArray, int size);

	/**
	 * The communications socket to talk to the driver
	 */
	int _socket;

	/**
	 * The interface name ("wlan0, wlan1, etc)
	 */
	QString _interfaceName;
	/**
	 * The current wireless mode
	 */
	WirelessMode _mode;
	/** Flag indicating whether or not WEP is enabled */
	bool _wepEnabled;
	/** The current WEP key */
	QString _wepKey;
	/** The ssid */
	QString _ssid;
	/** Wireless stats object */
	iwstats *_stats;
	/** Wireless range object Duncan: not used yet*/
	iwrange *_range;

};

#endif
